import java.applet.Applet;
import java.applet.AppletStub;
import java.awt.Dimension;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.nio.channels.Channels;
import java.nio.channels.ReadableByteChannel;
import java.util.Properties;

import javax.swing.JFrame;

/**
 * An {@link Applet} used for loading the rs2-client.
 * 
 * @author Thomas Le Godais <thomaslegodais@live.com>
 */
public class Loader extends Applet implements AppletStub {

	/**
	 * An generated serial UID.
	 */
	private static final long serialVersionUID = 1670498001014004354L;

	/**
	 * This client instance.
	 */
	private static Loader rsLoader;

	/**
	 * The parameters of the client.
	 */
	private Properties parameters = new Properties();

	/**
	 * The current frame of the client application.
	 */
	public JFrame clientFrame = null;

	/**
	 * Are we connecting to the Jagex Servers or Our Own?
	 */
	public static boolean isRSPS = false;

	/**
	 * This is the RSPS Server Address
	 */
	public static String SERVER_ADDRESS = "127.0.0.1";

	/**
	 * This is the RSPS Port
	 */
	public static final int PORT = 43594;

	/**
	 * This is the RSPS Port
	 */
	public static final boolean downloadParams = true;

	/**
	 * RSA KEYS
	 */

	/**
	 * The print stream instance.
	 */

	// private PrintStream printStream;

	/**
	 * The main entry point of the current application.
	 * 
	 * @param args
	 *            The command line arguments.
	 */
	public static void main(String... args) {
		rsLoader = new Loader();
		rsLoader.doFrame();
	}

	/**
	 * Performs the operations needed to show the client.
	 */
	private void doFrame() {
		readParameters();
		openFrame();
		startClient();
		clientFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	/**
	 * Starts the actual client.
	 */
	private void startClient() {
		try {

			Class<?> client_class = Class.forName("client");

			Object v_client = client_class.getConstructor().newInstance();
			client_class.getSuperclass().getMethod("supplyApplet", Applet.class).invoke(v_client, this);
			client_class.getMethod("init").invoke(v_client);
			client_class.getMethod("start").invoke(v_client);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Opens the actual frame application.
	 */
	private void openFrame() {
		clientFrame = new JFrame("Jagex");
		clientFrame.add(this);
		clientFrame.setVisible(true);
		clientFrame.setSize(1010, 720);
	}

	/**
	 * Reads the parameters text file, and stores the parameters.
	 */
	@SuppressWarnings("resource")
	private void readParameters() {
		if (isRSPS) {
			System.out.println("Loading RSPS");
			parameters.put("boxborder", "false");
			parameters.put("image", "http://www.runescape.com/img/game/splash4.gif");
			parameters.put("boxbgcolor", "black");
			parameters.put("centerimage", "true");
			parameters.put("separate_jvm", "true");
			parameters.put("java_arguments", "-Xmx384m -Xss2m -Dsun.java2d.noddraw=true -XX:CompileThreshold=1500 -Xincgc -XX:+UseConcMarkSweepGC -XX:+UseParNewGC");
			parameters.put("11", "DLRDVVWR5g39VImATeMlW298gWTaDNrx");
			parameters.put("0", "R9MxKEoV3hHbqpm2IWQ-kg");
			parameters.put("38", "false");
			parameters.put("1", "-908084124");
			parameters.put("29", SERVER_ADDRESS);
			parameters.put("31", "true");
			parameters.put("16", "1127");
			parameters.put("12", "false");
			parameters.put("9", "wwGlrZHF5gKN6D3mDdihco3oPeYN2KFybL9hUUFqOvk");
			parameters.put("26", "");
			parameters.put("14", "48");
			parameters.put("22", "");
			parameters.put("30", "");
			parameters.put("-1", "GagIWJzjx-NE86SvBSkRBg");
			parameters.put("35", "");
			parameters.put("3", ".runescape.com");
			parameters.put("10", "0");
			parameters.put("28", "1127");
			parameters.put("36", "1812221507");
			parameters.put("40", "0");
			parameters.put("8", "false");
			parameters.put("25", "");
			parameters.put("27", "");
			parameters.put("5", "false");
			parameters.put("23", "11396");
			parameters.put("21", "false");
			parameters.put("7", "0");
			parameters.put("17", "true,false,0,362,413,15,0,0,150,-15,Verdana,13,0xFFFFFF,rs3_login_loading_bar_whole.gif,rs3_login_loading_bar_back.gif,rs3_login_loading_bar_left.gif,rs3_login_loading_bar_right.gif,rs3_login_loading_bar_top.gif,rs3_login_loading_bar_bottom.gif,rs3_login_loading_bar_blackbox.gif,rs3_login_loading_bar_blackbox.gif,rs3_login_loading_bar_blackbox.gif,0");
			parameters.put("37", "225");
			parameters.put("34", "0");
			parameters.put("20", "http://world48.runescape.com/html5/client");
			parameters.put("4", SERVER_ADDRESS);
			parameters.put("13", "0");
			parameters.put("2", "false");
			parameters.put("15", "false");
			parameters.put("24", "http://services.runescape.com/m=gamelogspecs/clientstats?data=");
			parameters.put("33", "721353462");
			parameters.put("32", "0");
			parameters.put("haveie6", "true");
		} else {
			try {
				System.out.println("Loading RS");
				/**
				 * Grabs new params at each run :)
				 */
				if (downloadParams) {
					URL website = new URL("http://world48.runescape.com/html5/client");
					ReadableByteChannel rbc = Channels.newChannel(website.openStream());
					FileOutputStream fos = new FileOutputStream("params_download.txt");
					fos.getChannel().transferFrom(rbc, 0, Long.MAX_VALUE);
				}
				BufferedReader reader = new BufferedReader(new FileReader("./params_download.txt"));

				String line;
				while ((line = reader.readLine()) != null) {
					if (line.contains("document.write('<param name=")) {
						String[] regex = line.split(" value=");

						String parameterName = regex[0];
						String[] parameterRegex = parameterName.split("name=");
						String finalParameter = parameterRegex[1].replaceAll("\"", "");

						String parameterValue = regex[1];
						String finalValue = parameterValue.replaceAll("\"", "").replace(">');", "");

						parameters.put(finalParameter, finalValue);
						System.out.println("parameters.put(\"" + finalParameter + "\", \"" + finalValue + "\");");
					}
				}

			} catch (FileNotFoundException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.applet.AppletStub#appletResize(int, int)
	 */
	@Override
	public void appletResize(int dimensionX, int dimensionY) {
		super.resize(new Dimension(dimensionX, dimensionY));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.applet.Applet#getParameter(java.lang.String)
	 */
	@Override
	public String getParameter(String paramName) {
		return (String) parameters.get(paramName);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.applet.Applet#getDocumentBase()
	 */
	@Override
	public URL getDocumentBase() {
		try {
			if (!isRSPS) {
				return new URL("http://world61.runescape.com");
			} else {
				return new URL("http://" + SERVER_ADDRESS);
			}
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.applet.Applet#getCodeBase()
	 */
	@Override
	public URL getCodeBase() {
		try {
			if (!isRSPS) {
				return new URL("http://world61.runescape.com");
			} else {
				return new URL("http://" + SERVER_ADDRESS);
			}
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * If we should print out interfaces.
	 */
	public static final boolean PRINTING_INTERFACES = true;

	/**
	 * Prints out the current interface sent to the client.
	 * 
	 * @param interfaceId
	 *            The ID of the interface.
	 * @param interfacePosition
	 *            The position of the interface on the screen.
	 * @param walkable
	 *            If the interface can be walked around when opened.
	 */
	public void onInterface(int interfaceId, int interfacePosition, int walkable) {
		if (!PRINTING_INTERFACES) {
			return;
		}
		int windowID = interfacePosition >> 16;
		int slotID = interfacePosition & 0xFFFF;
		boolean isWalkable = (walkable == 1);
		System.out.println("sendInterface(" + interfaceId + ", " + windowID + ", " + slotID + ", " + isWalkable + ");");
	}

	/**
	 * If we should print out interface configs.
	 */
	public static final boolean PRINTING_INTERFACE_CONFIGS = true;

	/**
	 * Prints out the interface config sent to the client.
	 * 
	 * @param position
	 *            The position of the config.
	 * @param start
	 *            The start value of the config.
	 * @param end
	 *            The end value of the config.
	 * @param hash
	 *            The config hash.
	 */
	public void onInterfaceConfig(int position, int start, int end, int hash) {
		if (!PRINTING_INTERFACE_CONFIGS) {
			return;
		}
		int windowID = position >> 16;
		int slotID = position & 0xFFFF;
		System.out.println("interfaceConfig(" + windowID + ", " + slotID + ", " + start + ", " + end + ", " + hash + ");");
	}

	/**
	 * If we should print out configs or not.
	 */
	public static final boolean PRINTING_CONFIGS = true;

	/**
	 * Prints out a config sent to the client.
	 * 
	 * @param configID
	 *            The ID of the config.
	 * @param configValue
	 *            The value of the config.
	 */
	public void onConfig(int configID, int configValue) {
		if (!PRINTING_CONFIGS) {
			return;
		}
		System.out.println("Config(" + configID + ", " + configValue + ");");
	}

	/**
	 * If we're printing out client scripts.
	 */
	public static final boolean PRINTING_CLIENT_SCRIPTS = true;

	/**
	 * Prints out a client script sent to the client.
	 * 
	 * @param objects
	 *            The client script objects.
	 */
	public void onClientScript(Object[] objects) {
		if (!PRINTING_CLIENT_SCRIPTS) {
			return;
		}
		StringBuilder builder = new StringBuilder("ClientScript(");
		for (int index = 0; index < objects.length; index++) {
			builder.append(objects[index]);
			if (index != (objects.length - 1)) {
				builder.append(", ");
			}
		}
		builder.append(");");
		System.out.println(builder.toString());
	}

	/**
	 * If we're printing out CS2 Configs.
	 */
	public static final boolean PRINTING_CS2_CONFIGS = true;

	/**
	 * Prints out a cs2 config sent to the client.
	 * 
	 * @param configID
	 *            The ID of the config.
	 * @param configValue
	 *            The value of the config.
	 */
	public void onCS2Config(int configID, int configValue) {
		if (!PRINTING_CS2_CONFIGS) {
			return;
		}
		// printStream.println("CS2Config(" + configID + ", " + configValue +
		// ");");
		System.out.println("CS2Config(" + configID + ", " + configValue + ");");
	}

	/**
	 * Gets the instance of the client.
	 * 
	 * @return The instance.
	 */
	public static Loader getInstance() {
		return rsLoader;
	}
}